package com.linkv.rtcsdk;

import android.app.Application;
import android.view.ViewGroup;

import com.linkv.rtcsdk.bean.VideoQuality;
import com.linkv.rtcsdk.lvrtc.LVRTCLive;
import com.linkv.rtcsdk.utils.LogUtils;

import java.util.ArrayList;


/**
 * Created by Xiaohong on 2020/7/18.
 * desc:
 */
abstract public class LinkVRTCEngine {
    private static String TAG = "LinkVRTCEngine";

    // 3为自研 4为即构
    public static final int SDK_TYPE_LVRTC = 3;
    public static final int SDK_TYPE_ZEGO = 4;

    protected IEventHandler mIEventHandler = null;
    protected String mRoomId;
    protected int sdk_type;
    protected boolean mIsHost;
    protected String mUid = null;

    private static LinkVRTCEngine mEngine = null;
    // 记录当前拉流的userId
    private ArrayList<String> mPlayingUserIds = new ArrayList<>();

    /**
     * 创建音视频引擎,每次调用都会根据sdkType创建一个新的引擎对象，所以每次调用之后须重新进行初始化等工作。
     *
     * @param application          应用的Application
     * @param appID                后台申请的appId
     * @param appKey               后台申请的appKey
     * @param sdkType              常量SDK_TYPE_LVRTC为LinkV引擎，SDK_TYPE_ZEGO为即构引擎。
     * @param isDebugEnv           是否测试环境
     * @param isInternationalEnv   是否国际环境（仅linkv有效）
     * @param rtcInitHandler       音视频结果回调
     * @return 音视频引擎
     */
    public static LinkVRTCEngine createEngine(Application application, String appID, String appKey, int sdkType, boolean isDebugEnv, boolean isInternationalEnv, RtcInitHandler rtcInitHandler) {
        mEngine = new LVRTCLive();
        mEngine.initSDK(application, appID, appKey, isDebugEnv, isInternationalEnv, rtcInitHandler);
        return mEngine;
    }

    // 对外的监听器设置。
    public void setEventHandler(IEventHandler ieventhandler) {
        mIEventHandler = ieventhandler;
    }

    public LinkVRTCEngine() {
    }

    // 设置是否打开日志
    public void setLogOpen(boolean isOpen) {
        LogUtils.willLog = isOpen;
    }

    /**
     * 推流参数配置
     *
     * @param videoConfigLevel 推流参数预设值
     */
    public abstract void setVideoConfig(VideoConfigLevel videoConfigLevel);

    // 初始化
    protected abstract void initSDK(Application application, String appID, String appKey, boolean isDebugEnv, boolean isInternationalEnv, RtcInitHandler rtcInitHandler);

    // 进入房间
    public abstract void loginRoom(String uid, String roomId, boolean isHost, boolean isAudioOnly);

    // 退出房间
    public void logoutRoom() {
        // 停止推流和拉流，避免用户忘记调用
        stopPublishingStream();
        for (String playingUid : mPlayingUserIds) {
            stopPlayingStream(playingUid);
        }
    }

    // 开始预览

    /**
     * 本地摄像头预览
     *
     * @param userId               本地用户ID，不能为空串和null。
     * @param container            预览的控件容器
     * @param isZOrderMediaOverlay 因为是在surfaceView显示视频，此参数用来控制显示层级
     */
    public abstract void startPreview(String userId, ViewGroup container, boolean isZOrderMediaOverlay);

    // 开始推流
    public abstract void startPublishing();

    // 设置推流质量更新间隔
    public abstract void setPublishQualityMonitorCycle(int second);

    // 设置拉流质量更新间隔
    public abstract void setPlayQualityMonitorCycle(int second);

    /**
     * 拉流
     *
     * @param userId               拉流的ID
     * @param container            拉流显示的控件容器
     * @param isZOrderMediaOverlay 因为是在surfaceView显示视频，此参数用来控制显示层级
     */
    public abstract void startPlayingStream(String userId, ViewGroup container, boolean isZOrderMediaOverlay);


    /**
     * 停止推流
     */
    public abstract void stopPublishingStream();

    /**
     * 停止拉流
     *
     * @param userId 用户流ID
     */
    public abstract void stopPlayingStream(String userId);

    /**
     * 停止预览
     */
    public abstract void stopPreview();


    /**
     * 打开摄像头数据
     */
    public abstract void startCapture();

    /**
     * 关闭摄像头数据
     */
    public abstract void stopCapture();


    // 转换前后置摄像头
    public abstract void useFrontCamera(boolean mFrontCamera);

    // 获取连麦SDK的名称
    abstract protected String getSolutionName();

    // 获取支持连麦数
    abstract public int getSupportLine();

    // 获取SDK类型 1为声网，3为自研，4为zego。
    abstract public int getSDKType();

    // 静音推流
    abstract public void setMuteOutput(boolean mute);

    // 获取当前房间ID
    abstract public String getRoomId();

    // 释放引擎
    public abstract void unInitSDK();


    /**
     * 设置滤镜美颜级别
     *
     * @param beautyLevel 滤镜美颜级别 取值范围[0,1]
     */
    public abstract void setBeautyLevel(float beautyLevel);

    /**
     * 设置滤镜明亮度
     *
     * @param brightLevel 滤镜明亮度 取值范围[0,1]
     */
    public abstract void setBrightLevel(float brightLevel);

    /**
     * 设置滤镜饱和度
     *
     * @param toneLevel 滤镜饱和度级别 取值范围[0,1]
     */
    public abstract void setToneLevel(float toneLevel);


    // 远端用户房间回调。
    protected void onAddRemoterStream(String userId) {
        LogUtils.d(getSolutionName(), "onAddRemoterStream uid:" + userId);
        mIEventHandler.onRemoteStreamAdd(userId);
        mPlayingUserIds.add(userId);
    }

    // 远端用户离开房间
    protected void onRemoteStreamLeave(String uid) {
        LogUtils.d(getSolutionName(), "remoteLeave uid:" + uid);
        mIEventHandler.onRemoteStreamEnd(uid);
        mPlayingUserIds.remove(uid);
    }


    //// RTC SDK的version
    public abstract String getVersionName();

    /**
     * 被踢出房间或断开房间链接
     *
     * @param errorCode 错误码，自研或zego返回的。
     * @param roomID    房间ID
     */
    protected void onExitRoom(int errorCode, String roomID) {
        if (mIEventHandler != null) {
            mIEventHandler.onRoomDisconnect(errorCode, roomID);
        }
    }

    /**
     * 进入房间成功
     *
     * @param roomID 房间ID
     */
    protected void onLoginRoomSucceed(String roomID) {
        if (mIEventHandler != null) {
            mIEventHandler.onRoomConnected(roomID);
        }
    }


    protected void onVideoSizeChanged(String userId, int width, int height) {
        if (mIEventHandler != null) {
            mIEventHandler.onVideoSizeChanged(userId, width, height);
        }
    }

    public void onPublishStreamQualityUpdate(String userId, VideoQuality videoQuality) {
        if (mIEventHandler != null) {
            mIEventHandler.onPublisherQualityUpdate(userId, videoQuality);
        }
    }

    public void onRemoteQualityUpdate(String userId, VideoQuality videoQuality) {
        if (mIEventHandler != null) {
            mIEventHandler.onRemoteQualityUpdate(userId, videoQuality);
        }
    }

    public void invokePlayStateUpdate(int state, String userId) {
        if (mIEventHandler != null) {
            mIEventHandler.onPlayStateUpdate(state, userId);
        }
    }

    public void invokePublishStateUpdate(int state) {
        if (mIEventHandler != null) {
            mIEventHandler.onPublishStateUpdate(state);
        }
    }

    public interface IEventHandler {

        // 远端流停止
        void onRemoteStreamEnd(String userId);

        // 远端流加入
        void onRemoteStreamAdd(String userId);

        /**
         * 房间连接失败
         *
         * @param errorCode 错误码，文档中要分开写zego和自研的错误码。
         * @param roomID
         */
        void onRoomDisconnect(int errorCode, String roomID);

        // 房间连接成功
        void onRoomConnected(String roomID);

        // 远端流质量更新
        void onRemoteQualityUpdate(String streamID, VideoQuality quality);

        // 推流质量更新
        void onPublisherQualityUpdate(String streamID, VideoQuality quality);

        // 远端流视频宽高更新
        void onVideoSizeChanged(String streamID, int width, int height);

        /**
         * 推流状态更新
         *
         * @param state 0为推流成功，其它为失败或正在请求拉流
         */
        void onPublishStateUpdate(int state);

        /**
         * 拉流状态更新
         *
         * @param state  0为拉流成功，其它为失败或正在请求拉流
         * @param userId 拉流的ID
         */
        void onPlayStateUpdate(int state, String userId);
    }


    // RTC初始化结果回调
    public interface RtcInitHandler {
        int INIT_RESULT_SUCCEED = 0;
        int INIT_RESULT_FAIL = -1;

        void onRtcInitResult(int resultCode);
    }

    // 推流参数预设值
    public enum VideoConfigLevel {
        VIDEO_CONFIG_180P(0),
        VIDEO_CONFIG_270P(1),
        VIDEO_CONFIG_360P(2),
        VIDEO_CONFIG_540P(3),
        VIDEO_CONFIG_720P(4),
        VIDEO_CONFIG_1080P(5);

        int value;

        int value() {
            return value;
        }

        VideoConfigLevel(int value) {
            this.value = value;
        }
    }


}
